local ADDON_NAME = ...
local ChatTabsChannels = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "AceEvent-3.0", "AceHook-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceConsole = LibStub("AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME)

local function SetEditBoxChannelAttributes(editBox, channel)
	editBox:SetAttribute("chatType", channel)
	editBox:SetAttribute("stickyType", channel)
end

local function SetEditBoxCustomChannelAttributes(editBox, channelName)
	SetEditBoxChannelAttributes(editBox, "CHANNEL")
	editBox:SetAttribute("channelTarget", channelName)
end

local function SetEditBoxWhisperAttributes(editBox, whisperType, name)
	SetEditBoxChannelAttributes(editBox, whisperType)
	editBox:SetAttribute("tellTarget", name)
end

local function SetPartyAutoChannel(editBox)
	local channel
	local inInstance, _ = IsInInstance()
	if inInstance then
		channel = "INSTANCE_CHAT"
	elseif IsInRaid() then
		channel = "RAID"
	elseif IsInGroup() then
		channel = "PARTY"
	else
		channel = "SAY"
	end
	SetEditBoxChannelAttributes(editBox, channel)
end

local function SetCustomChannel(editBox, binding)
	SetEditBoxCustomChannelAttributes(editBox, binding.channel)
end

local function SetWhisperChannel(editBox)
	local lastTold, lastToldType = ChatEdit_GetLastToldTarget()
	if lastTold then
		SetEditBoxWhisperAttributes(editBox, lastToldType, lastTold)
	else
		local lastTell, lastTellType = ChatEdit_GetLastTellTarget()
		if lastTell then
			SetEditBoxWhisperAttributes(editBox, lastTellType, lastTell)
		end
	end
end

local function SetChannel(editBox, binding)
	SetEditBoxChannelAttributes(editBox, string.upper(binding.channelType))
end

local function UpdateEditBoxChannel(tabTitle, editBox)
	if not ChatTabsChannels.db.profile.enabled then
		return
	end
	
	local binding = ChatTabsChannels.db.profile.bindings[tabTitle]
	if binding and binding.enabled then
		local handlers = {
			["PARTY_AUTO"] = SetPartyAutoChannel,
			["CHANNEL"] = SetCustomChannel,
			["WHISPER"] = SetWhisperChannel
		}
		(handlers[string.upper(binding.channelType)] or SetChannel)(editBox, binding)
	end
end

local function ChatTabEditBoxEventHandler(editBox)
	local chatTab = _G[SELECTED_DOCK_FRAME:GetName() .. "Tab"]
	local tabTitle = chatTab:GetText()
	UpdateEditBoxChannel(tabTitle, editBox)
end

local function SetupChatTabsHooks()
	for i = 1, MAX_WOW_CHAT_CHANNELS do
		local editBox = _G["ChatFrame" .. i .. "EditBox"]
		ChatTabsChannels:HookScript(editBox, "OnEditFocusGained", ChatTabEditBoxEventHandler)
		ChatTabsChannels:HookScript(editBox, "OnMouseDown", ChatTabEditBoxEventHandler)
	end
end

local function CreateBindingOptions(tabTitle, binding, uiOrder)
	return {
		type = "group",
		name = tabTitle,
		order = uiOrder,
		args = {
			enable = {
				type = "toggle",
				name = L["EnableBinding"],
				get = function ()
					return binding.enabled
				end,
				set = function (info, value)
					binding.enabled = value
				end,
				order = 1;
			},
			separator = {
				type = "header",
				name = "",
				order = 10
			},
			deleteBinding = {
				type = "execute",
				name = L["DeleteBinding"],
				func = function ()
					local dialog = StaticPopup_Show("CHATTABSCHANNELS_DELETE_BINDING_CONFIRMATION", tabTitle)
					if dialog then
						dialog.bindingKey = tabTitle
					end
				end,
				order = 20
			}
		}
	}
end

local function FillOptionsWithExistingBindings(options)
	local maxOrder = 0
	for _, widget in pairs(options.args) do
		if widget.order > maxOrder then
			maxOrder = widget.order
		end
	end
	
	local bindings = ChatTabsChannels.db.profile.bindings
	local tabTitles = {}
	for tabTitle, _ in pairs(bindings) do
		table.insert(tabTitles, tabTitle)
	end
	table.sort(tabTitles, function (a, b)
		return string.lower(a) < string.lower(b)
	end)
	for i = 1, #tabTitles do
		local tabTitle = tabTitles[i]
		local binding = bindings[tabTitle]
		options.args["binding" .. i] = CreateBindingOptions(tabTitle, binding, maxOrder + i)
	end
end

local function CreateNewBinding()
	local params = ChatTabsChannels.newBindingParams
	if not params.tabTitle then
		StaticPopup_Show("CHATTABSCHANNELS_EMPTY_TAB_TITLE")
	elseif not params.channelType then
		StaticPopup_Show("CHATTABSCHANNELS_EMPTY_CHANNEL_TYPE")
	elseif params.channelType == "CUSTOM" and not params.channel then
		StaticPopup_Show("CHATTABSCHANNELS_EMPTY_CHANNEL_NAME")
	else
		if ChatTabsChannels.db.profile.bindings[params.tabTitle] then
			StaticPopup_Show("CHATTABSCHANNELS_BINDING_ALREADY_EXISTS", params.tabTitle)
			return
		end
		local newBinding = {
			enabled = true,
			channelType = params.channelType,
			channel = params.channel
		}
		ChatTabsChannels.db.profile.bindings[params.tabTitle] = newBinding
		ChatTabsChannels.newBindingParams = {}
	end
end

local function DeleteBinding(bindingKey)
	ChatTabsChannels.db.profile.bindings[bindingKey] = nil
	AceConfigRegistry:NotifyChange(ADDON_NAME)
end

local function GenerateOptions()
	local options = {
		type = "group",
		name = ADDON_NAME .. " v" .. GetAddOnMetadata(ADDON_NAME, "Version"),
		handler = ChatTabsChannels,
		args = {
			enable = {
				type = "toggle",
				name = L["EnableAddon"](ADDON_NAME),
				get = function ()
					return ChatTabsChannels.db.profile.enabled
				end,
				set = function (info, value)
					ChatTabsChannels.db.profile.enabled = value
				end,
				order = 1
			},
			createNewBindingHeader = {
				type = "header",
				name = L["NewBindingHeader"],
				order = 10
			},
			tabTitle = {
				type = "input",
				dialogControl = "EditBoxTabTitlePredictor",
				name = L["NewBindingTabTitle"],
				get = function ()
					return ChatTabsChannels.newBindingParams.tabTitle
				end,
				set = function (info, value)
					ChatTabsChannels.newBindingParams.tabTitle = value
				end,
				order = 20
			},
			channelType = {
				type = "select",
				name = L["NewBindingChannelType"],
				values = {
					PARTY_AUTO = L["ChannelTypePartyAuto"],
					PARTY = L["ChannelTypeParty"],
					RAID = L["ChannelTypeRaid"],
					INSTANCE_CHAT = L["ChannelTypeInstanceChat"],
					GUILD = L["ChannelTypeGuild"],
					WHISPER = L["ChannelTypeWhisper"],
					CUSTOM = L["ChannelTypeCustom"]
				},
				get = function ()
					return ChatTabsChannels.newBindingParams.channelType
				end,
				set = function (info, value)
					ChatTabsChannels.newBindingParams.channelType = value
				end,
				order = 30
			},
			channel = {
				type = "input",
				name = L["NewBindingChannelName"],
				hidden = function ()
					return ChatTabsChannels.newBindingParams.channelType ~= "CUSTOM"
				end,
				get = function ()
					return ChatTabsChannels.newBindingParams.channel
				end,
				set = function (info, value)
					ChatTabsChannels.newBindingParams.channel = value
				end,
				order = 40
			},
			create = {
				type = "execute",
				name = L["CreateNewBinding"],
				width = "half",
				func = CreateNewBinding,
				order = 50
			},
			existingBindingsHeader = {
				type = "header",
				name = L["ExistingBindingsHeader"],
				order = 60
			}
		}
	}
	
	FillOptionsWithExistingBindings(options)
	
	return options
end

local function SetupOptions()
	AceConfigRegistry:RegisterOptionsTable(ADDON_NAME, GenerateOptions)
	ChatTabsChannels.optionsFrame = AceConfigDialog:AddToBlizOptions(ADDON_NAME)
	
	local resetNewBindingParamsFunc = function ()
		ChatTabsChannels.newBindingParams = {}
	end
	ChatTabsChannels:HookScript(ChatTabsChannels.optionsFrame, "OnShow", resetNewBindingParamsFunc)
	ChatTabsChannels.optionsFrame.okay = resetNewBindingParamsFunc
	ChatTabsChannels.optionsFrame.cancel = resetNewBindingParamsFunc
	ChatTabsChannels.optionsFrame.default = resetNewBindingParamsFunc
	
	LibStub("AceGUI-3.0-Search-EditBox"):Register("TabTitlePredictor", function (self, text, values, max)
		local query = "^" .. string.lower(text)
		for i = 1, MAX_WOW_CHAT_CHANNELS do
			local tab = _G["ChatFrame" .. i .. "Tab"]
			if tab:IsVisible() then
				local tabTitle = tab:GetText()
				if string.match(string.lower(tabTitle), query) then
					values[tabTitle] = tabTitle
				end
			end
		end
	end)
end

local function InitDB()
	local defaults = {
		profile = {
			enabled = true,
			bindings = {}
		}
	}
	
	ChatTabsChannels.db = AceDB:New(ADDON_NAME .. "_DB", defaults, true)
end

local function InitStaticPopups()
	StaticPopupDialogs["CHATTABSCHANNELS_EMPTY_TAB_TITLE"] = {
		text = L["DialogEmptyTabTitle"],
		button1 = L["DialogButtonOk"],
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
	}
	
	StaticPopupDialogs["CHATTABSCHANNELS_EMPTY_CHANNEL_TYPE"] = {
		text = L["DialogEmptyChannelType"],
		button1 = L["DialogButtonOk"],
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
	}
	
	StaticPopupDialogs["CHATTABSCHANNELS_EMPTY_CHANNEL_NAME"] = {
		text = L["DialogEmptyChannelName"],
		button1 = L["DialogButtonOk"],
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
	}
	
	StaticPopupDialogs["CHATTABSCHANNELS_BINDING_ALREADY_EXISTS"] = {
		text = L["DialogBindingAlreadyExists"],
		button1 = L["DialogButtonOk"],
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
	}
	
	StaticPopupDialogs["CHATTABSCHANNELS_DELETE_BINDING_CONFIRMATION"] = {
		text = L["DialogDeleteBindingConfirmation"],
		button1 = L["DialogButtonYes"],
		button2 = L["DialogButtonCancel"],
		OnAccept = function (self)
			DeleteBinding(self.bindingKey)
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
	}
end

local function SetupSlashCommands()
	AceConsole:RegisterChatCommand("ctc", function ()
		InterfaceOptionsFrame_OpenToCategory(ChatTabsChannels.optionsFrame)
	end)
end

function ChatTabsChannels:OnInitialize()
	InitDB()
	self:RegisterEvent("PLAYER_ENTERING_WORLD", SetupChatTabsHooks)
	SetupOptions()
	InitStaticPopups()
	SetupSlashCommands()
end
